/***********************************************************************
Global variables and functions
************************************************************************/

/*---------------------------------------------------------------------
     Definition area. Write pin definition here.
----------------------------------------------------------------------*/
int SS_PIN    = 4;                      // D4ピンをHC595のラッチ信号（SPIのSS信号）
int SCK_PIN   = 7;                      // D7ピンをHC595のシフトクロック信号
int MOSI_PIN  = 8;                      // D8ピンをHC595のシリアル入力に
int VR_IN     = A0;                     // アナログ入力（半固定抵抗入力）

int ex_swPin  = A1;                     // A1ピンを外部スイッチ入力に
int swPin     = 26;                     // D26はオンボードSW_USER


const byte LED_SEGMENT[] =              // セグメント信号（負論理）
{ //ABCDEFGH  negative logic 0:ON/1:OFF
  0b00000011,   // segment data for 0
  0b10011111,   // segment data for 1
  0b00100101,   // segment data for 2
  0b00001101,   // segment data for 3
  0b10011001,   // segment data for 4
  0b01001001,   // segment data for 5
  0b01000001,   // segment data for 6
  0b00011111,   // segment data for 7
  0b00000001,   // segment data for 8
  0b00001001,   // segment data for 9
  0b11111111    // space
};

const byte LED_DIGIT[] =
{
  0x8F,   // select 1st digit from left
  0x4F,   // select 2nd digit from left
  0x2F,   // select 3rd digit from left
  0x1F    // select 4th digit from left
};

int old_time = 0x0000;                    // previous time(milli sec.)
int time_mode;

int analog_data;                          // 半固定抵抗器からの電圧の読み出し値
int analog_work;                          // 作業用
int ad_cnt  = 0;                          // 読み出し値の格納ポインタ
int analog_buff[4];                       // 読み出し値の格納バッファ
int analog_sum;                           // 4回分の合計値
byte blank_flag = 1;                      // 0サプレッション・フラグ
byte blank_mode = 1;                      // 0サブレス モード

char prescale4sw = 0;                     // SWチェックタイミングカウンタ( up to 5:20milli sec.)
int precount1 = 0x0000;                   // 100ミリ秒カウント用

int sw_data = 0xFFFF;                     // SW入力用

void setup() {
  // put your setup code here, to run once:
    pinMode(SS_PIN, OUTPUT);              // set D4pin to output mode
    pinMode(SCK_PIN, OUTPUT);             // set D7pin to output mode
    pinMode(MOSI_PIN, OUTPUT);            // set D8pin to output mode
    pinMode(ex_swPin, INPUT);             // set A1pin to input mode
    pinMode(swPin, INPUT);                // set D26pin to input mode

    digitalWrite( SCK_PIN, LOW ); // set SCK signal to LOW

//  アナログ入力の初期設定

    analogReference(DEFAULT);
    analog_data = analogRead(VR_IN);      // アナログ値を読みを出す
    analog_buff[0]  = analog_data;        // バッファの値を設定
    analog_buff[1]  = analog_data;
    analog_buff[2]  = analog_data;
    analog_buff[3]  = analog_data;
    analog_sum  = ( analog_data << 2 );   // 積算値の初期化
    
}

void loop() {
  // put your main code here, to run repeatedly:

    int time_work;                        // 現在時間のバッファ
    byte com_sel;                         // コモン信号用バッファ
    byte seg_data;                        // セグメント信号用バッファ
    byte sw_work;                         // SWの状態の作業用
    int  dt_wk;                           // データの作業用

/*--------------------------------------
    wait for 4milli seconds interval.
----------------------------------------*/

    time_work = ( int )( millis() & 0x0FFC ); // ミリ秒時間読み出し
    if ( old_time != time_work )          // 4ミリ秒経過か
    {

/*--------------------------------------
    4ミリ秒毎に表示を切り替える
----------------------------------------*/

        old_time = time_work;           // 現在時間を更新
        time_work &= 0x00C;             // 4ミリ秒毎の下位2桁をセット
    
        switch ( time_work )
        {
            case 0x00:                          // 1000の桁
                dt_wk = ( analog_data / 1000 ); // 1000の桁を求める
                if ( ( 1 == blank_flag ) && (0x00 == dt_wk ) )
                {
                  dt_wk = 10;                   // 0表示を消す
                }
                else
                {
                  blank_flag = 0;               // 0サプレスを解除
                }
                com_sel = LED_DIGIT[0];         // 桁選択信号をセット
                break;

            case 0x04:                          // 100桁選択信号をセット
                dt_wk = ( analog_data % 1000 ); // 1000の桁を削除
                dt_wk /= 100;                   // 100の桁を求める
                if ( ( 1 == blank_flag ) && (0x00 == dt_wk ) )
                {
                  dt_wk = 10;                   // 0表示を消す
                }
                else
                {
                  blank_flag = 0;               // 0サプレスを解除
                }
                com_sel = LED_DIGIT[1];         // 桁選択信号をセット
                break;

            case 0x08:                          // 10桁選択信号をセット
                dt_wk = ( analog_data % 100 );  // 100以上の桁を削除
                dt_wk /= 10;                    // 10の桁を求める
                if ( ( 1 == blank_flag ) && (0x00 == dt_wk ) )
                {
                  dt_wk = 10;                   // 0表示を消す
                }
                else
                {
                  blank_flag = 0;               // 0サプレスを解除
                }
                com_sel = LED_DIGIT[2];         // 桁選択信号をセット
                break;

            default:                            // 1の桁
                dt_wk = ( analog_data % 10 );   // 1の桁を求める
                blank_flag = blank_mode;        // 0サプレスを設定
                com_sel = LED_DIGIT[3];         // 桁選択信号をセット

        }

        seg_data = LED_SEGMENT[dt_wk];          // セグメント・データに変換

        digitalWrite( SS_PIN, LOW );            // SS signal(low level)をオンに

        shiftOut (MOSI_PIN, SCK_PIN, LSBFIRST, seg_data); // セグメント情報を送信

        shiftOut (MOSI_PIN, SCK_PIN, LSBFIRST, com_sel);  // 桁情報を送信

        digitalWrite( SS_PIN, HIGH );           // SS signalをオフ(rise up)

/*--------------------------------------
    every 4milli seconds timing
    if count enable, then count time up.
----------------------------------------*/

        if ( 25 <= ++precount1 )
        {                                       // 100ミリ秒経過
            precount1 = 0;                      // 4ミリ秒カウンタをクリア

/*--------------------------------------
 * 　最新の値を取り込む
 ----------------------------------------*/
            analog_data = analogRead(VR_IN);    // アナログ値を読みを出す
            analog_sum -= analog_buff[ad_cnt];  // もっとも古いデータを削除
            analog_sum += analog_data;          // 最新のデータを加算
            analog_buff[ad_cnt] = analog_data;  // 最新のデータを格納

            analog_data = ( analog_sum >> 2);   // 最新の平均値を設定
            ++ad_cnt;                           // ポインタ更新
            ad_cnt &= 0x03;                     // 折り返し
            
        }

/*--------------------------------------
    switch data check timing
----------------------------------------*/

        if ( ++prescale4sw >= 5 )       // check SW check timing or not
        {                               // 20milli seconds passing
            prescale4sw = 0x00;         // clear timing counter
            sw_data <<= 1;              // shift olddata left
            sw_data += ( digitalRead(ex_swPin)&digitalRead(swPin) ); // read sw data
            sw_work = (sw_data & 0x06); // extract check timing data
            if ( 0x04 == sw_work )
            {                           // sw is pushed
                blank_mode ^= 0x01;     // change blank mode flag
            }
        }
    
    }


}
